/*
 *  Copyright (C) 2010-2016. Mellanox Technologies, Ltd. ALL RIGHTS RESERVED.
 *
 *    Licensed under the Apache License, Version 2.0 (the "License"); you may
 *    not use this file except in compliance with the License. You may obtain
 *    a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 *    THIS CODE IS PROVIDED ON AN  *AS IS* BASIS, WITHOUT WARRANTIES OR
 *    CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT
 *    LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS
 *    FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
 *
 *    See the Apache Version 2.0 License for specific language governing
 *    permissions and limitations under the License.
 *
 */

#ifndef __SPECTRUM_H__
#define __SPECTRUM_H__
#include <resource_manager/resource_manager.h>

/************************************************
 *  Local Defines
 ***********************************************/

/************************************************
 *  Local Macros
 ***********************************************/

/************************************************
 *  Local Type definitions
 ***********************************************/

/************************************************
 *  Defines
 ***********************************************/

#define SPECTRUM_ACL_INGRESS_GROUPS_MAX       95    /* Backward compatibility */
#define SPECTRUM_ACL_ENGRESS_GROUPS_MAX       31    /* Backward compatibility */
#define SPECTRUM_ACL_GROUPS_SIZE_MAX          16    /* cap max acl_group_size */
#define SPECTRUM_ACL_GROUPS_NUM_MIN           0
#define SPECTRUM_ACL_GROUPS_NUM_MAX           400   /* cap max acl groups */
#define SPECTRUM_ACL_RULES_BLOCK_MAX          20    /* the same exist in resource_manager.h */
#define SPECTRUM_ACL_RULES_NUM_MIN            0
#define SPECTRUM_ACL_RULES_NUM_MAX            UNLIMITED_SDK_TABLE_SIZE /* cap_max_rules - 16k *//*UNILIMITED - temp for MLNX_OS*/
#define SPECTRUM_ACL_PBS_ENTRIES_MIN          0
#define SPECTRUM_ACL_PBS_ENTRIES_MAX          0xc000
#define SPECTRUM_ACL_PORT_RANGES_MAX          16    /* cap max_acl_l4_port_ranges */
#define SPECTRUM_ACL_REGIONS_MAX              400   /* cap_max_acl_regions */
#define SPECTRUM_ACL_INGRESS_TABLES_MAX       512   /* Backward compatibility */
#define SPECTRUM_ACL_EGRESS_TABLES_MAX        128   /* Backward compatibility */
#define SPECTRUM_ACL_TABLES_MAX               400   /* cap_max_acl_regions */
#define SPECTRUM_ACL_VLAN_GROUPS_MAX          255   /* cap_max_acl_vlan_groups  */
#define SPECTRUM_ACL_TRAP_GROUPS_MAX          3     /* not handled */
#define SPECTRUM_ACL_MAX_FLEX_KEYS            6     /* new - cap_flexible_keys */
#define SPECTRUM_ACL_MAX_ACTIONS_PER_RULE     16    /* new - cap_max_action_per_rule, see RM_API_ACL_MAX_ACTIONS_IN_RULE */
#define SPECTRUM_ACL_MAX_ACTIONS_PER_SET      3     /* new - cap_actions_per_set */
#define SPECTRUM_ACL_EXTENDED_ACTIONS_NUM_MIN 0
#define SPECTRUM_ACL_EXTENDED_ACTIONS_NUM_MAX UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_ACL_PARSING_DEPTH            96

#define SPECTRUM_ADJ_MC_NUM_MIN      0
#define SPECTRUM_ADJ_MC_NUM_MAX      UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_ERIF_LIST_NUM_MIN   0
#define SPECTRUM_ERIF_LIST_NUM_MAX   UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_RMID_NUM_MIN        0
#define SPECTRUM_RMID_NUM_MAX        UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_NVE_MC_LIST_NUM_MIN 0
#define SPECTRUM_NVE_MC_LIST_NUM_MAX UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_SMID_NUM_MIN        0
#define SPECTRUM_SMID_NUM_MAX        UNLIMITED_SDK_TABLE_SIZE

#define SPECTRUM_ROUTER_RIFS_MAX                      400
#define SPECTRUM_ROUTER_RIFS_DONTCARE                 (SPECTRUM_ROUTER_RIFS_MAX + 1)
#define SPECTRUM_ROUTER_SUB_PORT_FID_START_INDEX      0x3c00   /*15k*/
#define SPECTRUM_ROUTER_IPV4_UC_MAX                   UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_ROUTER_IPV4_MC_MAX                   UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_ROUTER_IPV6_UC_MAX                   UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_ROUTER_IPV6_MC_MAX                   UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_ROUTER_IPV4_MC_SYSTEM_MIN            0
#define SPECTRUM_ROUTER_IPV4_MC_SYSTEM_MAX            UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_ROUTER_IPV6_MC_SYSTEM_MIN            0
#define SPECTRUM_ROUTER_IPV6_MC_SYSTEM_MAX            UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_ROUTER_ECMP_MAX                      64
#define SPECTRUM_ROUTER_ECMP_HASH_MAX                 127
#define SPECTRUM_ROUTER_NEIGHS_MAX                    UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_ROUTER_ADJ_MAX                       0x1C000   /*112k*/
#define SPECTRUM_ROUTER_VRID_MAX                      254
#define SPECTRUM_ROUTER_COUNTERS_ID_MAX               0
#define SPECTRUM_ROUTER_NEXT_HOP_MAX                  64
#define SPECTRUM_ROUTER_ECMP_CONTAINER_TOT_WEIGHT_MAX 4096     /*4k*/
#define SPECTRUM_SHSPM_TREE_MAX                       24
#define SPECTRUM_SHSPM_ENTRY_MAX                      SPECTRUM_KVD_SIZE
#define SPECTRUM_MAX_ERIF_PER_RIGR                    9

#define SPECTRUM_IB_ROUTER_LIDS_MAX        0
#define SPECTRUM_IB_ROUTER_MC_LIDS_NUM_MAX 0
#define SPECTRUM_IB_ROUTER_UC_LID_NUM_MAX  0
#define SPECTRUM_IB_PKEY_TABLE_SIZE        0

#define SPECTRUM_FCF_PORT_DENSITY_MAX 0
#define SPECTRUM_FCF_ZONES_PAIRS_MAX  0
#define SPECTRUM_FCF_ZONES_GROUPS_MAX 0
#define SPECTRUM_FCF_GATEWAY_MAX      0
#define SPECTRUM_FCF_RULES_MAX        0

#define SPECTRUM_PORT_LCL_NUM_MAX             68
#define SPECTRUM_PORT_EXT_NUM_MAX             64
#define SPECTRUM_PORT_LOG_NUM_MAX             4096
#define SPECTRUM_PORT_MTU_MAX                 10000
#define SPECTRUM_PORT_MTU_MIN                 68
#define SPECTRUM_PORT_STORM_CONTROL_ID_MAX    4
#define SPECTRUM_PORT_VEPA_CHANNELS_NUM_MAX   0
#define SPECTRUM_PORT_SYSTEM_PORTS_MAX        64
#define SPECTRUM_PORT_SYSTEM_PORT_MODULES_MAX 64
#define SPECTRUM_IB_PORTS_MAX                 36

#define SPECTRUM_LAG_NUM_MAX          64
#define SPECTRUM_LAG_PORT_MEMBERS_MAX 32
#define SPECTRUM_LAG_VECTORS_NUM_MAX  0

#define SPECTRUM_SPAN_SESSION_ID_MAX_INTERNAL 2    /* supported by FW      */
#define SPECTRUM_SPAN_SESSION_ID_MAX_EXTERNAL 1    /* available to user    */

#define SPECTRUM_FDB_MID_MAX         7000
#define SPECTRUM_FDB_MID_START_INDEX 0
#define SPECTRUM_FDB_PGT_MAX         0
#define SPECTRUM_FDB_UC_ADDRESS_MAX  0x32000   /*200k*/
#define SPECTRUM_FDB_MC_ADDRESS_MAX  0x32000   /*200k*/

#define SPECTRUM_FLOODING_TABLE_MAX         2
#define SPECTRUM_FLOODING_TABLE_PER_VID_MAX 1

#define SPECTRUM_SWID_ID_MAX 0

#define SPECTRUM_POLICER_HOST_IFC_POOL_SIZE 32
#define SPECTRUM_POLICER_POOL_SIZE          1000 - SPECTRUM_POLICER_HOST_IFC_POOL_SIZE

#define SPECTRUM_POLICER_BS_MIN_VALUE_PACKETS    3
#define SPECTRUM_POLICER_BS_MIN_VALUE_BYTES      4
#define SPECTRUM_POLICER_BS_MAX_VALUE            30
#define SPECTRUM_POLICER_STORM_CONTROL_POOL_SIZE 320

#define SPECTRUM_COS_TRAFFIC_CLASS_MAX             15
#define SPECTRUM_COS_PORT_ETS_SUB_GROUP_MAX        7
#define SPECTRUM_COS_PORT_ETS_GROUP_MAX            7
#define SPECTRUM_COS_PORT_PRIO_MAX                 14
#define SPECTRUM_COS_PORT_COLOR_MAX                2
#define SPECTRUM_COS_PORT_ETS_ELEMENTS_NUM         35
#define SPECTRUM_COS_BUFFER_64K_SIZE_MAX           63.703125
#define SPECTRUM_COS_BUFFER_128K_SIZE_MAX          127.828125
#define SPECTRUM_COS_REDECN_DEFAULT_AQS_WEIGHT     5
#define SPECTRUM_COS_REDECN_RED_DROP_ENABLED       TRUE
#define SPECTRUM_COS_REDECN_SCD                    FALSE
#define SPECTRUM_COS_REDECN_PROFILES_MAX           3072
#define SPECTRUM_COS_REDECN_CELL_MULTIPLIER        64
#define SPECTRUM_COS_REDECN_DEFAULT_TC_RED_ENABLED FALSE
#define SPECTRUM_COS_REDECN_DEFAULT_TC_ECN_ENABLED FALSE
#define SPECTRUM_BRIDGE_NUM_MAX                    6144 /* 6k */
#define SPECTRUM_TOTAL_TCAM                        0x4000
#define SPECTRUM_TOTAL_L3_REPLICATION_MATRIX       0

#define SPECTRUM_NUM_HW_TRAP_GROUPS_MAX  32
#define SPECTRUM_TRAP_GROUP_PRIORITY_MIN 0
#define SPECTRUM_TRAP_GROUP_PRIORITY_MAX 5
#define SPECTRUM_NUM_RDQS_MAX            32
#define SPECTRUM_NUM_CPU_TCLASS          8

#define SPECTRUM_DECAP_RULES_NUM_MIN 0
#define SPECTRUM_DECAP_RULES_NUM_MAX UNLIMITED_SDK_TABLE_SIZE

#define SPECTRUM_IGMP_V3_NUM_MAX UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_IGMP_V3_NUM_MIN 0

/*SW TABLE -> HW TABLE MAPPING */

#define SPECTRUM_FDB_UC_MAC_HW_TABLE_TYPE /*sfd*/ RM_HW_TABLE_TYPE_TCAM_E
#define SPECTRUM_FDB_MC_MAC_HW_TABLE_TYPE /*sfd*/ RM_HW_TABLE_TYPE_TCAM_E

#define SPECTRUM_ROUTER_IPV4_UC_HW_TABLE_TYPE       /*ralue*/ RM_HW_TABLE_TYPE_KVD_HASH_E
#define SPECTRUM_ROUTER_IPV6_UC_HW_TABLE_TYPE       /*ralue*/ RM_HW_TABLE_TYPE_KVD_HASH_E
#define SPECTRUM_ROUTER_IPV4_MC_HW_TABLE_TYPE       /*rmft*/ RM_HW_TABLE_TYPE_TCAM_E
#define SPECTRUM_ROUTER_IPV6_MC_HW_TABLE_TYPE       /*rmft*/ RM_HW_TABLE_TYPE_TCAM_E
#define SPECTRUM_ROUTER_IPV4_MC_SYSTEM_HW_TABLE_TYPE/*rmft*/ (SPECTRUM_ROUTER_IPV4_MC_HW_TABLE_TYPE)
#define SPECTRUM_ROUTER_IPV6_MC_SYSTEM_HW_TABLE_TYPE/*rmft*/ (SPECTRUM_ROUTER_IPV6_MC_HW_TABLE_TYPE)

#define SPECTRUM_ROUTER_ARP_IPV4_HW_TABLE_TYPE/*rauht*/ RM_HW_TABLE_TYPE_KVD_HASH_E
#define SPECTRUM_ROUTER_ARP_IPV6_HW_TABLE_TYPE/*rauht*/ RM_HW_TABLE_TYPE_KVD_HASH_E

#define SPECTRUM_ROUTER_MC_REPLICATIONS_HW_TABLE_TYPE/*rigr*/ RM_HW_TABLE_TYPE_REPLICATION_MATRIX_E

#define SPECTRUM_ROUTER_ADJACENCY_HW_TABLE_TYPE/*ratr*/ RM_HW_TABLE_TYPE_KVD_LINEAR_E

#define SPECTRUM_L2_MC_MIDS_HW_TABLE_TYPE/*smid*/ RM_HW_TABLE_TYPE_TCAM_E

#define SPECTRUM_LAG_VECTORS_HW_TABLE_TYPE/*sldr*/ RM_HW_TABLE_TYPE_TCAM_E

#define SPECTRUM_L2_MC_FLOODING_HW_TABLE_TYPE/*sftr*/ RM_HW_TABLE_TYPE_TCAM_E

#define SPECTRUM_FCF_FORWORDING_HW_TABLE_TYPE/*fftr*/ RM_HW_TABLE_TYPE_TCAM_E

#define SPECTRUM_ACL_RULES_HW_TABLE_TYPE /*ptce*/ RM_HW_TABLE_TYPE_TCAM_E

#define SPECTRUM_ACL_EXTENDED_ACTIONS_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_LINEAR_E

#define SPECTRUM_ACL_PBS_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_LINEAR_E

#define SPECTRUM_ACL_GROUPS_HW_TABLE_TYPE  RM_HW_TABLE_TYPE_ACL_GROUP_TABLE_E
#define SPECTRUM_ADJ_MC_HW_TABLE_TYPE      RM_HW_TABLE_TYPE_KVD_LINEAR_E
#define SPECTRUM_ERIF_LIST_HW_TABLE_TYPE   RM_HW_TABLE_TYPE_KVD_LINEAR_E
#define SPECTRUM_RMID_HW_TABLE_TYPE        RM_HW_TABLE_TYPE_PGT_E
#define SPECTRUM_TUNNEL_RTDP_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_LINEAR_E
#define SPECTRUM_NVE_MC_LIST_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_LINEAR_E
#define SPECTRUM_SMID_HW_TABLE_TYPE        RM_HW_TABLE_TYPE_PGT_E

#define SPECTRUM_TUNNEL_RTDP_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_LINEAR_E
#define SPECTRUM_DECAP_RULES_HW_TABLE_TYPE RM_HW_TABLE_TYPE_TCAM_E

#define SPECTRUM_IGMP_V3_HW_TABLE_TYPE RM_HW_TABLE_TYPE_TCAM_E

/*phy_entry_cost per table */

#define SPECTRUM_FDB_UC_MAC_HW_ENTRY_COST 1
#define SPECTRUM_FDB_MC_MAC_HW_ENTRY_COST 1

#define SPECTRUM_ROUTER_IPV4_UC_HW_ENTRY_COST       /*ralue v4*/ 1
#define SPECTRUM_ROUTER_IPV6_UC_HW_ENTRY_COST       /*ralue v6 (depends on prefix length)*/ 1
#define SPECTRUM_ROUTER_IPV4_MC_HW_ENTRY_COST       /*rmft*/ 1
#define SPECTRUM_ROUTER_IPV6_MC_HW_ENTRY_COST       /*rmftv2*/ 2
#define SPECTRUM_ROUTER_IPV4_MC_SYSTEM_HW_ENTRY_COST/*rmft*/ (SPECTRUM_ROUTER_IPV4_MC_HW_ENTRY_COST)
#define SPECTRUM_ROUTER_IPV6_MC_SYSTEM_HW_ENTRY_COST/*rmftv2*/ (SPECTRUM_ROUTER_IPV6_MC_HW_ENTRY_COST)

#define SPECTRUM_ROUTER_ARP_IPV4_HW_ENTRY_COST/*rauht v4*/ 1
#define SPECTRUM_ROUTER_ARP_IPV6_HW_ENTRY_COST/*rauht v6*/ 2

#define SPECTRUM_ROUTER_MC_REPLICATIONS_HW_ENTRY_COST/*rigr*/ 1

#define SPECTRUM_ROUTER_ADJACENCY_HW_ENTRY_COST/*ratr*/ 1

#define SPECTRUM_L2_MC_MIDS_HW_ENTRY_COST/*smid*/ 1

#define SPECTRUM_LAG_VECTORS_ENTRY_COST 1

#define SPECTRUM_L2_MC_FLOODING_HW_ENTRY_COST/*sftr*/ 1

#define SPECTRUM_FCF_FORWORDING_HW_ENTRY_COST/*fftr*/ 1

#define SPECTRUM_ACL_RULES_HW_ENTRY_COST /*ptce*/ 1

#define SPECTRUM_ACL_EXTENDED_ACTIONS_HW_ENTRY_COST 1

#define SPECTRUM_ACL_PBS_HW_ENTRY_COST 1

#define SPECTRUM_ACL_GROUPS_HW_ENTRY_COST 1

#define SPECTRUM_ADJ_MC_HW_ENTRY_COST 1

#define SPECTRUM_ERIF_LIST_HW_ENTRY_COST 1

#define SPECTRUM_NVE_MC_LIST_HW_ENTRY_COST 1

#define SPECTRUM_TUNNEL_RTDP_HW_ENTRY_COST 1

#define SPECTRUM_DECAP_RULES_HW_ENTRY_COST 1

#define SPECTRUM_RMID_HW_ENTRY_COST 1

#define SPECTRUM_SMID_HW_ENTRY_COST    1
#define SPECTRUM_IGMP_V3_HW_ENTRY_COST 1

#define SPECTRUM_HEADROOM_NUM_BUFF_PER_PORT 9
/* Shared buffers - Condor */
#define SPECTRUM_BUFFERS_TOTAL_MEMORY    (16 * 1024 * 1024)
#define SPECTRUM_MAX_TOTAL_HEADROOM_SIZE (487 * 1024)

#define SPECTRUM_SHARED_BUFF_NUM_ING_POOL_PER_PORT 4
#define SPECTRUM_SHARED_BUFF_NUM_EGR_POOL_PER_PORT 4
#define SPECTRUM_SHARED_BUFF_TOTAL_NUM_POOL       \
    (SPECTRUM_SHARED_BUFF_NUM_ING_POOL_PER_PORT + \
     SPECTRUM_SHARED_BUFF_NUM_EGR_POOL_PER_PORT)
#define SPECTRUM_SHARED_BUFF_NUM_EGR_BUFF_PER_PORT  17
#define SPECTRUM_SHARED_BUFF_NUM_ING_BUFF_PER_PORT  9
#define SPECTRUM_SHARED_BUFF_MAX_POOL_SIZE          (16 * 1024 * 1024)
#define SPECTRUM_SHARED_BUFF_PORT_NUM_MAX           64
#define SPECTRUM_SHARED_BUFF_TCLASS_NUM_MAX         64
#define SPECTRUM_SHARED_BUFF_RET_STATISTICS_NUM_MAX 120
#define SPECTRUM_BUFF_UNIT_SIZE                     96
#define SPECTRUM_SHARED_BUFF_MC_MAX_NUM_PRIO        15

#define SPECTRUM_KVD_SIZE                 0x3C000          /*240K*/
#define SPECTRUM_KVD_HASH_SINGLE_MIN_SIZE 0x10000          /*64K*/
#define SPECTRUM_KVD_HASH_DOUBLE_MIN_SIZE 0x8000          /*16K*2 = 32K*/
#define SPECTRUM_KVD_GUARANTEED_CAPACITY  0.9          /*90%*/

#define SPECTRUM_PGT_SIZE  0x7000                      /* 28K */
#define SPECTRUM_RMPE_SIZE 0x3C00                       /* 15K */

/* SW Counter Allocation - Only used on some chips */
#define SPECTRUM_CNTR_BANK_PAIRS         4
#define SPECTRUM_CNTR_LINES_PER_BANK     4096
#define SPECTRUM_CNTR_LINES_FLOW_PKT     2
#define SPECTRUM_CNTR_TYPE_FLOW_PKT      0x03
#define SPECTRUM_CNTR_LINES_FLOW_BYTE    2
#define SPECTRUM_CNTR_TYPE_FLOW_BYTE     0x03
#define SPECTRUM_CNTR_LINES_FLOW_BOTH    2
#define SPECTRUM_CNTR_TYPE_FLOW_BOTH     0x03
#define SPECTRUM_CNTR_LINES_RIF_BASIC    10
#define SPECTRUM_CNTR_TYPE_RIF_BASIC     0x09
#define SPECTRUM_CNTR_LINES_RIF_MPLS     18
#define SPECTRUM_CNTR_TYPE_RIF_MPLS      0x0B
#define SPECTRUM_CNTR_LINES_RIF_IPV6     28
#define SPECTRUM_CNTR_TYPE_RIF_IPV6      0x0A
#define SPECTRUM_CNTR_LINES_RIF_EXTENDED 28
#define SPECTRUM_CNTR_TYPE_RIF_EXTENDED  0x0A

/* Tunnel */
#define SPECTRUM_NUM_MAX_TUNNEL_IPINIP 400      /* from SPECTRUM_ROUTER_RIFS_MAX */
#define SPECTRUM_NUM_MAX_TUNNEL_NVE    1
#define SPECTRUM_TUNNEL_RTDP_NUM_MIN   0
#define SPECTRUM_TUNNEL_RTDP_NUM_MAX   (SPECTRUM_NUM_MAX_TUNNEL_IPINIP + SPECTRUM_NUM_MAX_TUNNEL_NVE)

/* IGMP V3 */
#define SPECTRUM_IGMP_V3_NUM_MAX_ENTRIES 10000

/************************************************
 *  Macros
 ***********************************************/

/************************************************
 *  Type definitions
 ***********************************************/

/************************************************
 *  Global variables
 ***********************************************/
static const rm_resources_t rm_resource_spectrum = {
    /*13*/
    SPECTRUM_ACL_INGRESS_GROUPS_MAX,
    SPECTRUM_ACL_ENGRESS_GROUPS_MAX,
    SPECTRUM_ACL_GROUPS_SIZE_MAX,
    SPECTRUM_ACL_GROUPS_NUM_MAX,
    SPECTRUM_ACL_RULES_BLOCK_MAX,
    SPECTRUM_ACL_PBS_ENTRIES_MAX,
    SPECTRUM_ACL_PORT_RANGES_MAX,
    SPECTRUM_ACL_REGIONS_MAX,
    SPECTRUM_ACL_INGRESS_TABLES_MAX,
    SPECTRUM_ACL_EGRESS_TABLES_MAX,
    SPECTRUM_ACL_TABLES_MAX,
    SPECTRUM_ACL_VLAN_GROUPS_MAX,
    SPECTRUM_ACL_TRAP_GROUPS_MAX,
    SPECTRUM_ACL_MAX_FLEX_KEYS,
    SPECTRUM_ACL_MAX_ACTIONS_PER_RULE,
    SPECTRUM_ACL_MAX_ACTIONS_PER_SET,
    SPECTRUM_ACL_PARSING_DEPTH,

    /*20*/
    SPECTRUM_ROUTER_RIFS_MAX,
    SPECTRUM_ROUTER_RIFS_DONTCARE,
    SPECTRUM_ROUTER_SUB_PORT_FID_START_INDEX,
    SPECTRUM_ROUTER_IPV4_UC_MAX,
    SPECTRUM_ROUTER_IPV4_MC_MAX,
    SPECTRUM_ROUTER_IPV6_UC_MAX,
    SPECTRUM_ROUTER_IPV6_MC_MAX,
    SPECTRUM_ROUTER_ECMP_MAX,
    SPECTRUM_ROUTER_ECMP_HASH_MAX,
    SPECTRUM_ROUTER_ADJ_MAX,
    SPECTRUM_ROUTER_NEIGHS_MAX,
    SPECTRUM_ROUTER_VRID_MAX,
    SPECTRUM_ROUTER_COUNTERS_ID_MAX,
    SPECTRUM_ROUTER_NEXT_HOP_MAX,
    SPECTRUM_ROUTER_ECMP_CONTAINER_TOT_WEIGHT_MAX,
    SPECTRUM_SHSPM_TREE_MAX,
    SPECTRUM_SHSPM_ENTRY_MAX,

    /*5*/
    SPECTRUM_FCF_PORT_DENSITY_MAX,
    SPECTRUM_FCF_ZONES_PAIRS_MAX,
    SPECTRUM_FCF_ZONES_GROUPS_MAX,
    SPECTRUM_FCF_GATEWAY_MAX,
    SPECTRUM_FCF_RULES_MAX,

    /*8*/
    SPECTRUM_PORT_LCL_NUM_MAX,
    SPECTRUM_PORT_EXT_NUM_MAX,
    SPECTRUM_PORT_LOG_NUM_MAX,
    SPECTRUM_PORT_MTU_MAX,
    SPECTRUM_PORT_MTU_MIN,
    SPECTRUM_PORT_STORM_CONTROL_ID_MAX,
    SPECTRUM_PORT_VEPA_CHANNELS_NUM_MAX,
    SPECTRUM_PORT_SYSTEM_PORTS_MAX,
    SPECTRUM_PORT_SYSTEM_PORT_MODULES_MAX,

    /*6*/
    SPECTRUM_LAG_NUM_MAX,
    SPECTRUM_LAG_PORT_MEMBERS_MAX,

    SPECTRUM_SPAN_SESSION_ID_MAX_INTERNAL,
    SPECTRUM_SPAN_SESSION_ID_MAX_EXTERNAL,


    SPECTRUM_FDB_MID_MAX,
    SPECTRUM_FDB_MID_START_INDEX,
    SPECTRUM_FDB_PGT_MAX,
    SPECTRUM_FDB_UC_ADDRESS_MAX,
    SPECTRUM_FDB_MC_ADDRESS_MAX,

    SPECTRUM_FLOODING_TABLE_MAX,
    SPECTRUM_FLOODING_TABLE_PER_VID_MAX,

    SPECTRUM_COS_TRAFFIC_CLASS_MAX,
    SPECTRUM_COS_PORT_ETS_SUB_GROUP_MAX,
    SPECTRUM_COS_PORT_ETS_GROUP_MAX,
    SPECTRUM_COS_PORT_PRIO_MAX,
    SPECTRUM_COS_PORT_COLOR_MAX,
    SPECTRUM_COS_PORT_ETS_ELEMENTS_NUM,
    SPECTRUM_COS_BUFFER_64K_SIZE_MAX,
    SPECTRUM_COS_BUFFER_128K_SIZE_MAX,
    SPECTRUM_COS_REDECN_DEFAULT_AQS_WEIGHT,
    SPECTRUM_COS_REDECN_RED_DROP_ENABLED,
    SPECTRUM_COS_REDECN_SCD,
    SPECTRUM_COS_REDECN_PROFILES_MAX,
    SPECTRUM_COS_REDECN_CELL_MULTIPLIER,
    SPECTRUM_COS_REDECN_DEFAULT_TC_RED_ENABLED,
    SPECTRUM_COS_REDECN_DEFAULT_TC_ECN_ENABLED,
    SPECTRUM_SWID_ID_MAX,

    SPECTRUM_POLICER_POOL_SIZE,
    SPECTRUM_POLICER_HOST_IFC_POOL_SIZE,
    SPECTRUM_POLICER_STORM_CONTROL_POOL_SIZE,
    SPECTRUM_POLICER_BS_MIN_VALUE_PACKETS,
    SPECTRUM_POLICER_BS_MIN_VALUE_BYTES,
    SPECTRUM_POLICER_BS_MAX_VALUE,
    SPECTRUM_KVD_SIZE,
    SPECTRUM_KVD_HASH_SINGLE_MIN_SIZE,
    SPECTRUM_KVD_HASH_DOUBLE_MIN_SIZE,
    SPECTRUM_KVD_GUARANTEED_CAPACITY,

    SPECTRUM_PGT_SIZE,
    SPECTRUM_RMPE_SIZE,

    SPECTRUM_BRIDGE_NUM_MAX,

    SPECTRUM_BUFFERS_TOTAL_MEMORY,
    SPECTRUM_MAX_TOTAL_HEADROOM_SIZE,
    SPECTRUM_HEADROOM_NUM_BUFF_PER_PORT,
    /* Shared buffers - Condor */
    SPECTRUM_SHARED_BUFF_NUM_ING_POOL_PER_PORT,
    SPECTRUM_SHARED_BUFF_NUM_EGR_POOL_PER_PORT,
    SPECTRUM_SHARED_BUFF_TOTAL_NUM_POOL,
    SPECTRUM_SHARED_BUFF_NUM_EGR_BUFF_PER_PORT,
    SPECTRUM_SHARED_BUFF_NUM_ING_BUFF_PER_PORT,
    SPECTRUM_SHARED_BUFF_MAX_POOL_SIZE,
    SPECTRUM_SHARED_BUFF_PORT_NUM_MAX,
    SPECTRUM_SHARED_BUFF_TCLASS_NUM_MAX,
    SPECTRUM_SHARED_BUFF_RET_STATISTICS_NUM_MAX,
    SPECTRUM_BUFF_UNIT_SIZE,
    SPECTRUM_SHARED_BUFF_MC_MAX_NUM_PRIO,

    {
        {SPECTRUM_FDB_UC_ADDRESS_MAX,
         SPECTRUM_FDB_UC_ADDRESS_MAX,
         SPECTRUM_FDB_UC_MAC_HW_ENTRY_COST,
         SPECTRUM_FDB_UC_MAC_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_FDB_MC_ADDRESS_MAX,
         SPECTRUM_FDB_MC_ADDRESS_MAX,
         SPECTRUM_FDB_MC_MAC_HW_ENTRY_COST,
         SPECTRUM_FDB_MC_MAC_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_ROUTER_IPV4_UC_MAX,
         SPECTRUM_ROUTER_IPV4_UC_MAX,
         SPECTRUM_ROUTER_IPV4_UC_HW_ENTRY_COST,
         SPECTRUM_ROUTER_IPV4_UC_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_ROUTER_IPV6_UC_MAX,
         SPECTRUM_ROUTER_IPV6_UC_MAX,
         SPECTRUM_ROUTER_IPV6_UC_HW_ENTRY_COST,
         SPECTRUM_ROUTER_IPV6_UC_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_ROUTER_IPV4_MC_MAX,
         SPECTRUM_ROUTER_IPV4_MC_MAX,
         SPECTRUM_ROUTER_IPV4_MC_HW_ENTRY_COST,
         SPECTRUM_ROUTER_IPV4_MC_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_ROUTER_IPV6_MC_MAX,
         SPECTRUM_ROUTER_IPV6_MC_MAX,
         SPECTRUM_ROUTER_IPV6_MC_HW_ENTRY_COST,
         SPECTRUM_ROUTER_IPV6_MC_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_ROUTER_NEIGHS_MAX,
         SPECTRUM_ROUTER_NEIGHS_MAX,
         SPECTRUM_ROUTER_ARP_IPV4_HW_ENTRY_COST,
         SPECTRUM_ROUTER_ARP_IPV4_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_ROUTER_NEIGHS_MAX,
         SPECTRUM_ROUTER_NEIGHS_MAX,
         SPECTRUM_ROUTER_ARP_IPV6_HW_ENTRY_COST,
         SPECTRUM_ROUTER_ARP_IPV6_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_ROUTER_IPV4_MC_MAX + SPECTRUM_ROUTER_IPV6_MC_MAX,
         SPECTRUM_ROUTER_IPV4_MC_MAX + SPECTRUM_ROUTER_IPV6_MC_MAX,
         SPECTRUM_ROUTER_MC_REPLICATIONS_HW_ENTRY_COST,
         SPECTRUM_ROUTER_MC_REPLICATIONS_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_ROUTER_ADJ_MAX,
         SPECTRUM_ROUTER_ADJ_MAX,
         SPECTRUM_ROUTER_ADJACENCY_HW_ENTRY_COST,
         SPECTRUM_ROUTER_ADJACENCY_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_FDB_MID_MAX,
         SPECTRUM_FDB_MID_MAX,
         SPECTRUM_L2_MC_MIDS_HW_ENTRY_COST,
         SPECTRUM_L2_MC_MIDS_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_LAG_VECTORS_NUM_MAX,
         SPECTRUM_LAG_VECTORS_NUM_MAX,
         SPECTRUM_LAG_VECTORS_ENTRY_COST,
         SPECTRUM_LAG_VECTORS_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_FLOODING_TABLE_MAX,
         SPECTRUM_FLOODING_TABLE_MAX,
         SPECTRUM_L2_MC_FLOODING_HW_ENTRY_COST,
         SPECTRUM_L2_MC_FLOODING_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_FCF_RULES_MAX,
         SPECTRUM_FCF_RULES_MAX,
         SPECTRUM_FCF_FORWORDING_HW_ENTRY_COST,
         SPECTRUM_FCF_FORWORDING_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_ACL_RULES_NUM_MIN,
         SPECTRUM_ACL_RULES_NUM_MAX,
         SPECTRUM_ACL_RULES_HW_ENTRY_COST,
         SPECTRUM_ACL_RULES_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_ACL_EXTENDED_ACTIONS_NUM_MIN,
         SPECTRUM_ACL_EXTENDED_ACTIONS_NUM_MAX,
         SPECTRUM_ACL_EXTENDED_ACTIONS_HW_ENTRY_COST,
         SPECTRUM_ACL_EXTENDED_ACTIONS_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_ACL_GROUPS_NUM_MIN,
         SPECTRUM_ACL_GROUPS_NUM_MAX,
         SPECTRUM_ACL_GROUPS_HW_ENTRY_COST,
         SPECTRUM_ACL_GROUPS_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_ACL_PBS_ENTRIES_MIN,
         SPECTRUM_ACL_PBS_ENTRIES_MAX,
         SPECTRUM_ACL_PBS_HW_ENTRY_COST,
         SPECTRUM_ACL_PBS_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_ADJ_MC_NUM_MIN,
         SPECTRUM_ADJ_MC_NUM_MAX,
         SPECTRUM_ADJ_MC_HW_ENTRY_COST,
         SPECTRUM_ADJ_MC_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_ERIF_LIST_NUM_MIN,
         SPECTRUM_ERIF_LIST_NUM_MAX,
         SPECTRUM_ERIF_LIST_HW_ENTRY_COST,
         SPECTRUM_ERIF_LIST_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_TUNNEL_RTDP_NUM_MIN,
         SPECTRUM_TUNNEL_RTDP_NUM_MAX,
         SPECTRUM_TUNNEL_RTDP_HW_ENTRY_COST,
         SPECTRUM_TUNNEL_RTDP_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_DECAP_RULES_NUM_MIN,
         SPECTRUM_DECAP_RULES_NUM_MAX,
         SPECTRUM_DECAP_RULES_HW_ENTRY_COST,
         SPECTRUM_DECAP_RULES_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_ROUTER_IPV4_MC_SYSTEM_MIN,
         SPECTRUM_ROUTER_IPV4_MC_SYSTEM_MAX,
         SPECTRUM_ROUTER_IPV4_MC_SYSTEM_HW_ENTRY_COST,
         SPECTRUM_ROUTER_IPV4_MC_SYSTEM_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_ROUTER_IPV6_MC_SYSTEM_MIN,
         SPECTRUM_ROUTER_IPV6_MC_SYSTEM_MAX,
         SPECTRUM_ROUTER_IPV6_MC_SYSTEM_HW_ENTRY_COST,
         SPECTRUM_ROUTER_IPV6_MC_SYSTEM_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_RMID_NUM_MIN,
         SPECTRUM_RMID_NUM_MAX,
         SPECTRUM_RMID_HW_ENTRY_COST,
         SPECTRUM_RMID_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_NVE_MC_LIST_NUM_MIN,
         SPECTRUM_NVE_MC_LIST_NUM_MAX,
         SPECTRUM_NVE_MC_LIST_HW_ENTRY_COST,
         SPECTRUM_NVE_MC_LIST_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_SMID_NUM_MIN,
         SPECTRUM_SMID_NUM_MAX,
         SPECTRUM_SMID_HW_ENTRY_COST,
         SPECTRUM_SMID_HW_TABLE_TYPE,
         FALSE},

        {SPECTRUM_IGMP_V3_NUM_MIN,
         SPECTRUM_IGMP_V3_NUM_MAX,
         SPECTRUM_IGMP_V3_HW_ENTRY_COST,
         SPECTRUM_IGMP_V3_HW_TABLE_TYPE,
         FALSE},
    },

    {
        {TRUE, SPECTRUM_TOTAL_TCAM},
        {FALSE, 0},
        {FALSE, 0}, /*kvd hash size should be configured by user size*/
        {FALSE, 0}, /*kvd linear size should be configured by user size */
        {FALSE, 0},
        {FALSE, 0},
        {TRUE, SPECTRUM_ACL_GROUPS_NUM_MAX},
        {FALSE, 0}, /* PGT size should be configured by user size */
    },

    /*16*/
    SPECTRUM_CNTR_BANK_PAIRS,
    SPECTRUM_CNTR_LINES_PER_BANK,
    SPECTRUM_CNTR_LINES_FLOW_PKT,
    SPECTRUM_CNTR_TYPE_FLOW_PKT,
    SPECTRUM_CNTR_LINES_FLOW_BYTE,
    SPECTRUM_CNTR_TYPE_FLOW_BYTE,
    SPECTRUM_CNTR_LINES_FLOW_BOTH,
    SPECTRUM_CNTR_TYPE_FLOW_BOTH,
    SPECTRUM_CNTR_LINES_RIF_BASIC,
    SPECTRUM_CNTR_TYPE_RIF_BASIC,
    SPECTRUM_CNTR_LINES_RIF_MPLS,
    SPECTRUM_CNTR_TYPE_RIF_MPLS,
    SPECTRUM_CNTR_LINES_RIF_IPV6,
    SPECTRUM_CNTR_TYPE_RIF_IPV6,
    SPECTRUM_CNTR_LINES_RIF_EXTENDED,
    SPECTRUM_CNTR_TYPE_RIF_EXTENDED,

    /*10*/
    SPECTRUM_IB_PKEY_TABLE_SIZE,

    SPECTRUM_IB_ROUTER_LIDS_MAX,
    SPECTRUM_IB_ROUTER_MC_LIDS_NUM_MAX,
    SPECTRUM_IB_ROUTER_UC_LID_NUM_MAX,

    /*1*/
    SPECTRUM_IB_PORTS_MAX,

    /*5*/
    SPECTRUM_NUM_HW_TRAP_GROUPS_MAX,
    SPECTRUM_TRAP_GROUP_PRIORITY_MIN,
    SPECTRUM_TRAP_GROUP_PRIORITY_MAX,
    SPECTRUM_NUM_RDQS_MAX,
    SPECTRUM_NUM_CPU_TCLASS,

    /* Tunnel */
    SPECTRUM_NUM_MAX_TUNNEL_IPINIP,
    SPECTRUM_NUM_MAX_TUNNEL_NVE,

    /* IGMP V3 */
    SPECTRUM_IGMP_V3_NUM_MAX_ENTRIES,
};

/************************************************
 *  Function declarations
 ***********************************************/
#endif /*__SPECTRUM_H__*/
